/*
{*******************************************************************}
{                                                                   }
{       KS.CRM                                                      }
{       Customer Relationship Management                            }
{                                                                   }
{       Copyright (c) 2002 - 2013 by Kroll-Software                 }
{       All Rights Reserved                                         }
{                                                                   }
{   This code is provided 'as is',                                  }
{   without warranties of any kind. Use at your own risk.           }
{                                                                   }
{   You need a KS.CRM source-code license in order to use           }
{   these sources or portions of it.                                }
{                                                                   }
{   You must not publish, redistribute or transfer                  }
{   this source code to 3rd parties under any conditions.           }
{                                                                   }
{*******************************************************************}
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using KS.Common;
using KS.CRM.UI;

namespace KS.CRM.Favorites
{
    public partial class FavoriteForm : CrmEditFormBase
    {
        private int pRefID = 0;        

        public FavoriteForm()
            : base()
        {
            this.PlugInKey = "Favorites";
            this.HelpVisible = false;

            InitializeComponent();

            this.ucCategoriesSelect1.RootNodeTitle = Properties.Resources.CategoryRootNodeTitle;
            this.ucCategoriesSelect1.CategoriesChanged += new EventHandler(ucCategoriesSelect1_CategoriesChanged);

            this.capTitle.Caption = Properties.Resources.CaptionFavoriteTitle;
            this.capRemarks.Caption = Properties.Resources.CaptionRemarks;
            this.capCategories.Caption = Properties.Resources.CaptionCategories;            

            this.cmdOK.Click += new EventHandler(cmdOK_Click);
            this.panelBottom.BackColor = Helpers.GuiStyle.EditDialogBackColor;
        }

        void ucCategoriesSelect1_CategoriesChanged(object sender, EventArgs e)
        {
            m_CategoriesChanged = true;
        }

        protected bool m_CategoriesChanged = false;
        public bool CategoriesChanged
        {
            get
            {
                return m_CategoriesChanged;
            }
        }

        protected override void OnLayout(LayoutEventArgs levent)
        {
            base.OnLayout(levent);
            splitContainer1.SplitterDistance = this.crmTableLayoutPanel1.PreferredSize.Height;
        }        

        protected override void OnLoad(EventArgs e)
        {            
            base.OnLoad(e);
            this.LoadSettings(this.Name);
        }

        protected override void OnShown(EventArgs e)
        {
            base.OnShown(e);            

            this.txtDescription.Focus();
        }        

        public override void InitData(KS.CRM.Base.CrmDatabase DB)
        {
            base.InitData(DB);
            this.ucCategoriesSelect1.IsPrivate = true;
            this.ucCategoriesSelect1.InitData(m_DB, (int)KS.CRM.Base.BL.Common.TableTypes.Favorites);
        }

        public bool AddFavorite(int RefID, int TableID, string Title)
        {
            this.Text = Properties.Resources.FavFormAddToFavorites;                       

            m_ID = 0;
            pRefID = RefID;
            this.TableID = TableID;
            this.txtTitle.Text = Title;

            // Schon vorhanden ??
            System.Data.SqlClient.SqlDataReader dr = null;
            string strSQL = "select * from Favorites WHERE (UserID=" + m_DB.UserID + ") AND (RefID=" + pRefID.ToString() + ") AND (TableID=" + TableID.ToString() + ")";

            int FoundID = 0;

            try
            {
                dr = m_DB.ExecuteSQL(strSQL);

                if (dr.Read())
                {
                    FoundID = dr["ID"].SafeInt();
                    this.txtDescription.Text = dr["Description"].SafeString();
                }
            }
            catch (Exception ex)
            {
                Helpers.ErrMsgBox(ex.Message);
            }
            finally
            {
                if (dr != null)
                {                    
                    dr.Dispose();
                }
            }

            if (FoundID > 0)
            {
                switch (Helpers.QuestionMsgBox(Properties.Resources.FavFormConfirmOverwrite))
                {
                    case DialogResult.Yes:
                        //this.Text = "Favoriten bearbeiten";
                        //this.pID = FoundID;
                        //this.bNew = false;
                        this.Edit(FoundID);
                        return true;                        

                    case DialogResult.No:
                        return false;                        

                    case DialogResult.Cancel:
                        return false;        
                }                                    
            }

            this.ucCategoriesSelect1.LoadData(m_ID, "Category2Favorite");

            return true;
        }        

        public override bool Edit(int ID)
        {
            this.Text = Properties.Resources.FavFormCaptionEdit;
            base.Edit(ID);

            //this.ucCategoriesSelect1.IsPrivate = true;
            //if (!this.ucCategoriesSelect1.InitData(m_DB, (int)KS.CRM.Base.BL.Common.TableTypes.Favorites))
            //    return false;            

            // Favoriten laden..
            KS.CRM.Base.BL.Favorites cFavorites = new KS.CRM.Base.BL.Favorites(m_DB);

            if (cFavorites.GetFavorite(m_ID))
            {
                this.TableID = cFavorites.TableID;
                this.pRefID = cFavorites.RefID;
                this.txtTitle.Text = cFavorites.Title;
                this.txtDescription.Text = cFavorites.Description;

                this.txtTitle.Modified = false;
                this.txtDescription.Modified = false;
            }
            
            if (!this.ucCategoriesSelect1.LoadData(m_ID, "Category2Favorite"))
                return false;

            return true;
        }

        public override bool IsValid()
        {
            if (txtTitle.Text.Trim() == "")
            {
                Helpers.ErrMsgBox(Properties.Resources.FavFormTitleMissing);
                txtTitle.Focus();
                return false;
            }

            if (!ucCategoriesSelect1.IsValid()) return false;

            return true;
        }

        // This is what we usually implement for saving
        // this form does save a special ID, so we use our own function below
        //protected override bool SaveData()
        //{
        //    return base.SaveData();
        //}

        private bool SaveFavorite()
        {
            this.DialogResult = System.Windows.Forms.DialogResult.Abort;            

            bool bUpdatedFlag = false;
            bool bSetModifiedFlag = false;            

            m_Modified = txtTitle.Modified || txtDescription.Modified;

            KS.CRM.Base.BL.Favorites cFavorites = null;

            try
            {
                m_DB.BeginTransaction();
            }
            catch (Exception ex)
            {
                Helpers.ErrMsgBox(ex.Message);
                return false;
            }

            try
            {
                if (m_Modified || m_ID == 0)
                {
                    cFavorites = new KS.CRM.Base.BL.Favorites(m_DB);

                    cFavorites.TableID = this.TableID;
                    cFavorites.RefID = this.pRefID;
                    cFavorites.UserID = m_DB.UserID;
                    cFavorites.Title = this.txtTitle.Text.Trim();
                    cFavorites.Description = this.txtDescription.Text.Trim();

                    if (m_ID == 0)
                        m_ID = cFavorites.InsertFavorite();
                    else
                        cFavorites.UpdateFavorite(m_ID);

                    bUpdatedFlag = true;
                }

                if (m_ID > 0)
                {
                    bSetModifiedFlag = false;
                    ucCategoriesSelect1.SaveData(m_ID);                        

                    //if (bSetModifiedFlag)
                    //{
                    //    m_DB.ExecuteUpdateSQL("update Favorites Set changed=getDate() WHERE ID=" + m_ID.ToString());
                    //}
                }

                m_DB.CommitTransaction();
                
            }
            catch (Exception ex)
            {
                m_DB.RollbackTransaction();

                Helpers.ErrMsgBox(ex.Message);
                return false;
            }
            finally
            {
                if (cFavorites != null)
                    cFavorites.Dispose();
            }

            if (bUpdatedFlag || bSetModifiedFlag)
                this.DialogResult = System.Windows.Forms.DialogResult.OK;

            return true;
        }

        private void cmdOK_Click(object sender, EventArgs e)
        {            
            if (!IsValid())
                return;

            if (!SaveFavorite())
                return;

            if (this.DialogResult == DialogResult.None)
                this.DialogResult = System.Windows.Forms.DialogResult.OK;

            this.Close();
        }
    }
}